#############################################################################
# ClearCase/SiteConfig.pm -- ClearCase Site-specific Configuration Settings
#
# Copyright (C) 1995-2000 Brad Appleton. All rights reserved.
#
# This file is part of "ClearPerl" version 5 (ClearPerl5).
# This file/module is free software; you can redistribute it and/or
# modify it under the same terms as ClearPerl itself, as stated in
# the README file of the ClearPerl distribution.
#############################################################################


=head1 NAME

ClearCase::SiteConfig - ClearCase site-specific configuration settings

=head1 SYNOPSIS

  use ClearCase::Config;  ## automatically uses SiteConfig if needed

=head1 REQUIRES/IMPORTS

perl5.004

=head1 EXPORTS

Nothing by default, local sites may export whatever constants, variables,
and functions they choose to add. If they do, they should document them
in this section.

=head1 DESCRIPTION

C<ClearCase::SiteConfig> contains all the environment and site-specific
configuration settings and attributes for the version of ClearCase
running on the host system. The platform-specific settings for all
of these things are in C<ClearCase::Config>.  These platform-specific
settings may be extended and/or overridden using C<ClearCase::SiteConfig>.

C<ClearCase::SiteConfig> can override a setting in C<ClearCase::Config>
simply by providing its own implementation of one of the C<clearcase_*>
functions defined by the latter module. If C<ClearCase::SiteConfig> has
it's own implementation of one of hese functions, it will be invoked
I<instead of> the same-named function in C<ClearCase::Config>.

C<ClearCase::SiteConfig> can provide any additional functions or
variables it chooses simply by having the local site implement them,
then export them, and properly document them.

This allows local sites to customize their installation of ClearPerl
without having to constantly merge-in changes from subsequent versions
of C<ClearCase::Config>. They simply use <ClearCase::SiteConfig> instead
and the former promises to make use of the latter if its available.

=cut

#############################################################################

package ClearCase::SiteConfig;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

use Exporter;
use Carp;

@ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

#!# @EXPORT = qw(  );
#!# @EXPORT_OK = qw(  );
#!# %EXPORT_TAGS = ( 'all' => [@EXPORT, @EXPORT_OK] );

$VERSION = 5.000_013;
require  5.004;    ## requires this Perl version or later

#!# use ClearCase::OSUtil qw( );

#############################################################################
## Want to have access to ClearCase::Config, but DO NOT do a 'use' because
## we want to avoid any cyclic dependencies (since we know it depends on us).

require ClearCase::Config
    unless (defined &ClearCase::Config::_clearcase_config);
import ClearCase::Config qw( /^_clearcase_/ );

##---------------------------------------------------------------------------
## Uncomment and implement the following to override one or more
## configuration value settings.  Invoke _clearcase_config() to
## use the normal/default platform-specific settings for a
## configuration value
##---------------------------------------------------------------------------

#!# sub clearcase_config {
#!# }

##---------------------------------------------------------------------------
## Uncomment and implement the following to override one or more program
## path settings.  Invoke _clearcase_progpath() to use the normal/default
## platform-specific settings for a program-path
##---------------------------------------------------------------------------

#!# sub clearcase_progpath {
#!# }


##---------------------------------------------------------------------------
## Uncomment and implement the following to override one or more program
## version settings.  Invoke _clearcase_version() to to use the
## normal/default platform-specific settings for a program-version
##---------------------------------------------------------------------------

#!# sub clearcase_version {
#!# }


##---------------------------------------------------------------------------
## Uncomment and implement the following to override one or more program
## patch-level settings.  Invoke _clearcase_patches() to use the
## normal/default platform-specific settings for a patch-level
##---------------------------------------------------------------------------

#!# sub clearcase_patches {
#!# }


#############################################################################

=head1 AUTHORS

Brad Appleton  F<E<lt>bradapp@enteract.comE<gt>>

=head1 SEE ALSO

ClearCase::Config, ClearCase::OSUtil

=cut

1;

__END__
