# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

use strict;

# FIXME: some VOB path. If cleartool is as above, pwd works fine.
#
my $VOB = '/usr/vob/testvob';

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)
my $loaded;
BEGIN {
  $| = 1; print "1..21\n";
}
END {
  print "not ok 1\n" unless $loaded;
}

use ClearCase::ClearTool;
use ClearCase::Config qw(clearcase_config);
$loaded = 1;
print "ok 1\n";
my $VERSION = $ClearCase::ClearTool::VERSION;

# Automatically generates an ok/nok msg, incrementing the test number.
my( $rc, $allok );
{
   my $next = 2;
   sub ok {
      my $status = shift;
      my $skip   = shift;
      $skip = defined( $skip ) ? ' # skip' : '';
      return ($status ? '' : 'not ') . "ok @{[$next++]}$skip\n";
   }

   # use this to put a version check guard on a test
   sub req {
      my( $version, $test ) = @_;
      if( $VERSION >= $version ){
        &$test;
      } else {
        print ok( 0, 'skip' ); 
      }
   }
}

######################### End of black magic.

# FIXME: No need to have ClearCase to test.
# All we need to run this test is some dir X/bin containing a cleartool:
#     #! /usr/bin/sh
#     "$@"
# and export ATRIAHOME=X. Tests are run using /bin/ls on the working dir.
# Errors are provoked by describing invalid file

my $cchome = clearcase_config('ATRIA_HOME');
my $ctpath =  clearcase_config('CLEARTOOL');
if (! -d $cchome) {
   my $wv = qx($ctpath pwv);
   if ($? || ! $wv) {
      print "\nNo ClearCase found on this system, skipping tests ...\n";
      exit( 0 );
   }
}

# subcommand used for testing (Note: $ct->lsvob(...) also used below.)
#
my( @Cmd, @Opt, @Stdout, @StdoutChomp, $Stdout );
@Cmd = @Opt = qw( lsvob -l );
shift( @Opt );

# find a VOB we can play with
$VOB = ""  unless (-d $VOB);
open( CT, "$ctpath lsvob $VOB |" )  or  die( "cannot read from cleartool" );
$VOB = "";
while ( <CT> ) {
    ## Find an existing vobtag directory of a mounted VOB
    next unless /^\*\s+(\S+)\s+/;
    last if (-d ($VOB = $1));
}
close( CT );
$VOB  or  die( "cannot find vobtag for an existing mounted VOB" );

# get subcommand result without using ClearTool
#
open( CT, "$ctpath @Cmd $VOB |" ) || die( "cannot read from cleartool" );
@Stdout = <CT>;
close( CT );
chomp( @StdoutChomp = @Stdout );
chomp( $Stdout = join( '', @Stdout ) );

###
### Here we go...
###
$allok = 1;

### Test 2: check that we get joined standard output
req 0,
sub {
  my $out = cleartool( @Cmd, $VOB );
  $rc = $out eq $Stdout;
  print ok( $rc );
  $allok &&= $rc;
};

### Test 3: check that we get standard output as a chomped array
req 0,
sub {
  my @out = cleartool( [-chomping], @Cmd, $VOB );
  $rc = join( '', @out ) eq join( '', @StdoutChomp );
  print ok( $rc );
  $allok &&= $rc;
};

### Test 4: check that we get standard output as an UNchomped array
req 0,
sub {
  my @out = cleartool( ['-nochomping'], @Cmd, $VOB );
  $rc = join( '', @out ) eq join( '', @Stdout );
  print ok( $rc );
  $allok &&= $rc;
};

### Test 5: same, use single option syntax
req 0,
sub {
  my @out = cleartool( '+nochomping', @Cmd, $VOB );
  $rc = join( '', @out ) eq join( '', @Stdout );
  print ok( $rc );
  $allok &&= $rc;
};

### Test 6: just get the status
req 0,
sub {
  my $st1a = cleartool( ['-status', '-stdout' => undef], @Cmd, $VOB );
  my $st1b = cleartool->status();
  my $st2a = cleartool( [ '-returns' => 'status', '-stderr' => undef ],
                        @Cmd, $VOB.'__' );
  my $st2b = cleartool->status();
  $rc =  $st1a == 0 && $st1b == 0  &&  $st2a != 0 && $st2b != 0;
  print ok( $rc );
  $allok &&= $rc;
};

### Test 7: just get the success value, different way of suppressing output
req 0,
sub {
  my $ok1a = cleartool( ['-success', '-nostdout' ], @Cmd, $VOB );
  my $ok1b = cleartool->success();
  my $ok2a = cleartool( ['-success', '-nostderr' ], @Cmd, $VOB.'__' );
  my $ok2b = cleartool->success();
  $rc = $ok1a && $ok1b  &&  ! $ok2a && ! $ok2b;
  print ok( $rc );
  $allok &&= $rc;
};

### Test 8: read it line by line
req 0,
sub {
  my $fh = cleartool( '+open', @Cmd, $VOB );
  my( $line, $i, $rc );
  $i = 0;
  $rc = 1;
  while( defined( $line = <$fh> ) ){
    $rc &= $line eq $Stdout[$i++];
  }
  $fh->close();
  $rc &= $i == @Stdout;
  print ok( $rc );
  $allok &&= $rc;
};

### Test 9: bad option
req 0,
sub {
  my $fh;
  eval { $fh = cleartool( '+qoen', @Cmd, $VOB ); };
  ### print $@;
  $rc = defined( $@ );
  print ok( $rc );
  $allok &&= $rc;
};

### Test 10: Does abort work in connection with open()?
req 0,
sub {
  my $fh = cleartool( [ '-open', '-abort' => "error in: @Cmd" ], @Cmd, $VOB.'__' );
  eval { $fh->close(); };
  $rc = defined( $@ ) && $@ =~ /^error in: @Cmd/;
  print ok( $rc );
  $allok &&= $rc;
};

### Test 11: Does abort with message work?
req 0,
sub {
  my $out;
  eval { $out = cleartool( [ '-abort' => "error in: @Cmd" ], @Cmd, $VOB.'__' ); };
  $rc = defined( $@ ) && $@ =~ /^error in: @Cmd/;
  print ok( $rc );
  $allok &&= $rc;
};


### Test 12: Does abort call user supplied function?
req 5.000_013,
sub {
  my $err = 0;
  my $msg;
  cleartool( [ '-abort' => sub { $msg = shift; $err++; } ], @Cmd, $VOB.'__' );
  $rc = $err == 1 && defined $msg;
  print ok( $rc );
  $allok &&= $rc;
};

### Test 13: Error message and user supplied function
req 5.000_013,
sub {
  my $err = 0;
  my $msg;
  cleartool( [ '-abort' => { '-errmsg' => "error in: @Cmd",
                             '-errsub' => sub { $msg = shift; $err++; } } ],
             @Cmd, $VOB.'__' );
  $rc = $err == 1 && defined( $msg ) && $msg =~ /^error in: @Cmd/;
  print ok( $rc );
  $allok &&= $rc;
};
 
### Test 14: Position of -abort: is missing argument correctly determined
req 0,
sub {
  my $cta = ClearCase::ClearTool->new( [-abort, -nochomping ] );
  my $resa;
  eval { $resa = $cta->lsvob( @Opt, $VOB.'__' ); };
  my $erra = defined $@ ? $@ : '';
  my $resb;
  my $ctb = ClearCase::ClearTool->new( [-abort ] );
  eval { $resb = $ctb->lsvob( @Opt, $VOB.'__' ); };
  my $errb = defined $@ ? $@ : '';
  $rc   = ! defined( $resa ) && ! defined( $resb );	
  $rc &&= $erra eq $errb;
  print ok( $rc );
  $allok &&= $rc;
};


### Test 15: chomped output() from explicit ClearTool object 
req 0,
sub {
  no strict "refs";
  my $ct =  ClearCase::ClearTool->new();
  my $outa = $ct->lsvob( @Opt, $VOB );
  my $outb = $ct->lsvob( '+nochomping', @Opt, $VOB );
  $rc = $outa eq $Stdout  &&  $outb eq "$Stdout\n";
  print ok( $rc );
  $allok &&= $rc;
};

### Test 16: UNchomped output() from explicit ClearTool object 
req 0,
sub {
  no strict "refs";
  my $ct =  ClearCase::ClearTool->new( [ '-nochomping' ] );
  my @outa = $ct->lsvob( @Opt, $VOB );
  my @outb = $ct->lsvob( '+chomping', @Opt, $VOB );
  my @outc = $ct->lsvob( @Opt, $VOB );
  $rc = join( '', @outa ) eq join( '', @Stdout )  &&
        join( '', @outb ) eq join( '', @StdoutChomp )  &&
        join( '', @outc ) eq join( '', @Stdout );
  print ok( $rc );
  $allok &&= $rc;
};

### Test 17: output() with explicit ClearTool object 
req 0,
sub {
  my $ct =  ClearCase::ClearTool->new( [ '-chomping' ] );
  my @outa = $ct->output( '+chomping', @Cmd, $VOB );
  my @outb = $ct->output( '+nochomping', @Cmd, $VOB );
  my @outc = $ct->output( @Cmd, $VOB );
  $rc = join( '', @outa ) eq join( '', @StdoutChomp )  &&
        join( '', @outb ) eq join( '', @Stdout )  &&
        join( '', @outc ) eq join( '', @StdoutChomp );
  print ok( $rc );
  $allok &&= $rc;
};

### Test 18: check status() and success() both ways
req 5.000_013,
sub {
  my $ct =  ClearCase::ClearTool->new();
  $ct->output( @Cmd, $VOB );
  my $sys1 = $?;
  my $sta1 = $ct->status();
  my $suc1 = $ct->success();

  $ct->output( [-abort=>undef()], @Cmd, $VOB.'__' );

  cleartool( @Cmd, $VOB.'__' );

  my $sys2 = $?;
  my $sta2 = $ct->status();
  my $suc2 = $ct->success();

  $rc = $sys1 == 0 && $sys1 == $sta1 && $suc1
        &&
        $sys2 != 0 && $sys2 == $sta2 && ! $suc2;

  print ok( $rc );
  $allok &&= $rc;
};

### Test 19: open() in two concurrent ClearTools, one with two streams
req 0,
sub {
  my $cta = ClearCase::ClearTool->new();
  my $fha = $cta->open( @Cmd, $VOB );
  my $ctb = ClearCase::ClearTool->new();
  my $fhb = $ctb->open( @Cmd, $VOB );
  my $fhc = $ctb->open( @Cmd, $VOB );

  my( $la, $lb, $lc );
  while( defined( $la = <$fha> ) ){
    $lb = <$fhb>;
    $lc = <$fhc>;
    $rc &= $la eq $lb && $lb eq $lc;
  }
  # check for excess lines in streams b, c
  $lb = <$fhb>;
  $rc &= ! defined( $lb );
  $lc = <$fhc>;
  $rc &= ! defined( $lc );

  # close (not tested here)
  $fha->close();
  $fhb->close();
  $fhc->close();

  print ok( $rc );
  $allok &&= $rc;
};

### Test 20: open() on a subcommand with output AND error
req 0,
sub {
  use integer;
  my $cta = ClearCase::ClearTool->new( '+nochomping' );
  my $fha = $cta->open( @Cmd, $VOB, '/no/such/vob__' );

  $rc = 1;

  ## do some fancy reading, check against UNchomped Stdout
  my $n = @Stdout/2;
  my $i;
  for( $i = 0; $i < $n; $i++ ){
    my $line = <$fha>;
    $rc &= defined( $line ) && $line eq $Stdout[$i];
  }
  ## get the 2nd half in one go
  my @remlin = <$fha>;
  $rc &= join( '', @remlin ) eq join( '', @Stdout[$i..$#Stdout] );

  ## close - to set status
  my $cst = $fha->close();

  ## must have nonzero status and false success
  my $sta = $cta->status();
  $rc &= $sta != 0;
  ### print "status returns: $sta\n";
  ### print "close  returns: $cst\n";

  my $oka = $cta->success();
  $rc &= ! $oka;

  print ok( $rc );
  $allok &&= $rc;
};

### Test 21: Wildcard expansion in subcommands
req 0,
sub {
  my $cmd = 'describe';
  my $vob = '*';

  my $ct = ClearCase::ClearTool->new( '+nochomping' );
  my @ls = $ct->output( $cmd, $vob );
  my $suca = $ct->success();
  my $fh   = $ct->open( $cmd, $vob );
  my $sucb = $ct->success();
  my $succ = $ct->success( [-nostdout], $cmd, $vob );
  my $stat = $ct->status( [-nostdout], $cmd, $vob );
  my $sucd = $ct->success();
  if( @ls ){
    $rc = $suca && $sucb && $succ && $sucd;
  } else {
    $rc = 0;
  }
  print ok( $rc );
  $allok &&= $rc;
};

### print STDERR "TheEnd ($allok)\n";
exit ($allok ? 0 : 1);

