#!/usr/local/bin/perl

use Config;
use File::Basename qw(&basename &dirname);

# List explicitly here the variables you want Configure to
# generate.  Metaconfig only looks for shell variables, so you
# have to mention them as if they were shell variables, not
# %Config entries.  Thus you write
#  $startperl
# to ensure Configure will look for $Config{startperl}.

# This forces PL files to create target in same directory as PL file.
# This is so that make depend always knows where to find PL derivatives.
chdir(dirname($0));
($file = basename($0)) =~ s/\.PL$//;
$file =~ s/\.pl$//
        if ($^O eq 'VMS' or $^O eq 'os2');  # "case-forgiving"

open OUT,">$file" or die "Can't create $file: $!";

print "Extracting $file (with variable substitutions)\n";

# In this section, perl variables will be expanded during extraction.
# You can use $Config{...} to use Configure variables.

print OUT <<"!GROK!THIS!";
$Config{'startperl'}
    eval 'exec perl -S \$0 "\$@"'
        if 0;
!GROK!THIS!

# In the following, perl variables are not expanded during extraction.

print OUT <<'!NO!SUBS!';
#
# dumbtool - dummy replacement program for cleartool(1) for use
#            in testing ClearCase/ClearTool.pm
#
use 5.004;
use File::Basename;

my $NAME = basename($0);

my $SYNOPSIS =
         "$NAME [-exit *status*] [-errmsg *text*] [-outmsg *text*]
                 [-signal *signame*] arg1 arg2 ...";

my $OPTIONS = "\
  -help
      Print a brief help-message and exit
  -exit *status*
      Specify the integer exit-status to exit with
  -errmsg *text*
      Specify text to print to STDERR (may be specified multiple times
      to issue multiple message lines)
  -outmsg *text*
      Specify text to print to STDOUT (may be specified multiple times
      to issue multiple message lines)
  -signal *signame* 
      Specify the name of a signal to send to the program
";

sub usage {
  my $rc = shift || 2;
  my $usage_msg = "Usage: $SYNOPSIS\n\nOptions/Arguments:$OPTIONS";
  if ($rc < 2) {
     print STDOUT $usage_msg;

  }
  else {
     print STDERR $usage_msg;
  }
  exit($rc);
}

use Getopt::Long;

## Make sure we allow normal cleartool options to "pass-through"
Getopt::Long::config qw(require_order pass_through);

my %OPTS = (
    'help' => 0,     ## print a help message
    'exit' => 0,     ## exit-status to use
    'errmsg' => [],  ## text to print to stderr
    'outmsg' => [],  ## text to print to stdout
    'signal' => 0,   ## name of signal to send to this program
);
my $rc = GetOptions( \%OPTS,
                     'help',
                     'exit=i',
                     'errmsg=s@',
                     'outmsg=s@', 
                     'signal=s'
         );
&usage(1) if $OPTS{'help'};
&usage(2) unless $rc;

my $status = $OPTS{'exit'};
my @output = @{ $OPTS{'outmsg'} };
my @errors = @{ $OPTS{'errmsg'} };
my $signal = uc $OPTS{'signal'};

## print command-line arguments
if (@output == 0) {
   @output = (@ARGV > 0) ? map("$NAME: ARGV[$_] = \"$ARGV[$_]\"", 0 .. $#ARGV)
                         : ("$NAME: NO command-line arguments given!");
}

for (@output) {
   chomp;
   print STDOUT "$_\n";
}

for (@errors) {
   chomp;
   print STDERR "$_\n";
}

if ($signal) {
   $signal =~ s/^SIG//;
   if ( $signal =~ /^[-+]?\d+$/ ) {
      print "\n+ $NAME: Sending signal #$signal to $NAME(pid=$$)\n";
      kill $signal, $$;
   }
   else {
      print "\n+ $NAME: Sending signal SIG$signal to $NAME(pid=$$)\n";
      kill "$signal", $$;
   }
}

print "\n+ $NAME: exiting with status $status\n";
exit($status);

!NO!SUBS!

close OUT or die "Can't close $file: $!";
chmod 0755, $file or die "Can't reset permissions for $file: $!\n";
exec("$Config{'eunicefix'} $file") if $Config{'eunicefix'} ne ':';
