# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl test.pl'

######################### We start with some black magic to print on failure.

# Change 1..1 below to 1..last_test_to_print .
# (It may become useful if the test is moved to ./t subdirectory.)

BEGIN { $| = 1; print "1..5\n"; }
END {print "not ok 1\n" unless $loaded;}
use ClearCase::OSUtil qw(:all);
use ClearCase::Config qw(clearcase_config clearcase_progpath);

$loaded = 1;
print "ok 1\n";

# Automatically generates an ok/nok msg, incrementing the test number.
{
   my $next = 2;
   sub ok {
      my $status = shift;
      return ($status ? '' : 'not ') . "ok @{[$next++]}\n";
   }
}

######################### End of black magic.

my ($rc, $final);

## Test - splitpathlist() and joinpathlist()
my @pnames = qw(alpha beta gamma delta);
my $path;

## Test - progname() and progpath()

## Test -  env4cli() and quote4cli()

## Test - devnull()

my $stdnull = MSWIN ? 'NUL' : '/dev/null';
my $devnull = devnull();
print ok($stdnull eq $devnull);

## Test - tmpdir() and tmpname()

$rc = 0;
my @tempfiles;
for (qw(xxx yyy zzz)) { push(@tempfiles, tmpname($_)); $rc = 1 if -f $_ }
print "Successfully generated ", scalar @tempfiles, " filenames: @tempfiles\n"
   unless $rc;
print ok($rc == 0);
$final |= $rc;

## Test - redirect_fh()
redirect_fh(STDOUT => 'off', STDERR => 'off');
print "You _should_ be seeing this message to STDOUT ...\n"; # uses SAVE_STDOUT
print STDOUT "You SHOULD NOT be seeing this message to STDOUT ...\n";
print STDERR "You SHOULD NOT be seeing this message to STDERR ...\n";
redirect_fh(STDOUT => 'on', STDERR => 'on');
print STDERR "You _should_ be seeing this message to STDERR ...\n";
print "Did you see any messages which say they SHOULD NOT be seen? [n] ";
$rc = <STDIN> =~ /^y/i;
print "Did you see exactly 2 messages which say they _should_ be seen? [y] ";
$rc |= <STDIN> =~ /^n/i;
print ok($rc == 0);
$final |= $rc;

## Test - redirect_fh(STDOUT => file)
my $log = tmpname('log');
redirect_fh(STDOUT => ">$log");
print STDOUT "Printing to STDOUT\n";
print STDERR "Printing to STDERR\n";
redirect_fh(STDOUT => 'ON');
my $lines = 0;
if (open(LOG, $log)) {
   while(<LOG>) { $lines++ }
   close(LOG);
}
print ok($lines == 1);
$final |= ($lines != 1);
unlink $log unless $final;

# Test - clearcase_progpath()
my $ct = clearcase_progpath 'cleartool';
$rc = system($ct, qw(pwd -h));
print ok($rc == 0);
$final |= $rc;

# Test - clearcase_config('CLEARTOOL');
my $CT = clearcase_config 'CLEARTOOL';
my $pwv_h = qx($CT pwv -h);
print ok($? == 0);

##
##  No test for mkattr().
##

##
##  No test for sendmsg().
##

exit ($final != 0);
