## This script is used to test the ClearCase::Config module.

use lib(qw(. ..));
use strict;

use ClearCase::Config qw(:all);
use ClearCase::OSUtil qw(:all);

# Automatically generates an ok/nok msg, incrementing the test number.
{
   my($next, @msgs);
   sub printok($) {
      my $okay = shift || 0;
      push(@msgs, ($okay ? '' : 'not ') . "ok @{[++$next]}\n");
      printf "end %d\n", scalar(@msgs);
   }
   sub testnum {1 + @msgs;}
   END {
      print "\n1..", scalar @msgs, "\n\n", @msgs;
   }
}

my (@names, @values, %names, %values, $i);

%values = clearcase_config;
foreach (sort keys %values)
{
   my $okay = /^CLEARCASE_(VIEWDRIVE|WINRGY|MVFSPARMS)/ || length $values{$_};
   printf "   Key: %-20s  Value: %s\n", $_, $okay ? $values{$_} : "undef";
   printok $okay;
}

print '
----------------------------------------------------------------------
Printing the value clearcase_config returns for \'CLEARTOOL\' when
called in a scalar context:

';

my $ctpath = clearcase_config('CLEARTOOL');
print "   CLEARTOOL yields: $ctpath\n";
printok progpath($ctpath);

print '
----------------------------------------------------------------------
Printing the values clearcase_config returns for the list of names

';
@names = qw(CLEARCASE_BIN ClearTool CC_BIN cc_xn_sfx ct CC_ViewRoot);
foreach (@names)
{
   print "   $_\n";
}
print '
when called in a scalar content.  This also tests if clearcase_config
understands how to translate short names and mixed cases:

';

@values = clearcase_config(@names);
for ($i = 0; $i < @names; $i++)
{
   printf "   Key: %-20s  Value: %s\n", $names[$i], defined $values[$i]
         ? $values[$i] : "undef";
   my $okay = ($names[$i] =~ /ViewRoot|xn_sfx/) ? length($values[$i])
                                                : (progpath $values[$i]);
   printok $okay;
}

print '
----------------------------------------------------------------------
Printing the values clearcase_version returns for the list of names

';
@names = qw(ClearCase MVFS cleartool clearmake
      abe view_server vobrpc_server db_server credmap_server db_dumper
      db_loader promote_server scrubber vob_scrubber);
      
foreach (@names)
{
   print "   $_\n";
}
print '
when called in an array content.  If clearcase_config above worked in
scalar and array contexts, then so does clearcase_version.

';

@values = clearcase_version(@names);
for ($i = 0; $i < @names; $i++)
{
   printf "   Key: %-20s  Value: %s\n", $names[$i], defined $values[$i]
         ? $values[$i] : "undef";
   printok  length($values[$i]);
}

print '
----------------------------------------------------------------------
Printing the values clearcase_patches returns for the list of names

';
@names = qw(clearcase_p3.2.1-6 p3.2.1-6 3.2.1-6);
      
foreach (@names)
{
   print "   $_\n";
}
print '
when called in an array content.  If clearcase_config above worked in
scalar and array contexts, then so does clearcase_version.

';

@values = clearcase_patches(@names);
for ($i = 0; $i < @names; $i++)
{
   printf "   Key: %-20s  Value: %s\n", $names[$i], defined $values[$i]
         ? $values[$i] : "undef";
}

print '
----------------------------------------------------------------------
';
@values = clearcase_patches;
foreach (sort @values)
{
   printf "   patch: %-20s\n", defined($_) ? $_ : 'undef';
}

